
from buildhat import Motor
import time

class rubiksCube:

   def __init__(self, motorFunction, motorRotate):
      self.motorFunction = motorFunction
      self.motorRotate = motorRotate

      self.gearsFunction = 1/(5/27) # 5.4
      self.gearsRotate = 1/((5/27) * (12/20)) # 9.0

      self.function = -1
      self.position = -1

      rubiksCube.initFunction(self)
      rubiksCube.rotatePosition(self)
      rubiksCube.initRotateClockWise(self)


   def initFunction(self):
      stepsize = 7
      oldpos=motorFunction.get_position()
      while True:
         motorFunction.run_for_degrees(stepsize * self.gearsFunction, speed=30, blocking=True)
         newpos=motorFunction.get_position()
         moved = abs(newpos-oldpos) / self.gearsFunction
         if moved < 3:
            break
         oldpos=newpos
      motorFunction.run_for_degrees(stepsize * self.gearsFunction, speed=-30, blocking=True)
      self.function = 0


   def initRotateClockWise(self):
      stepsize = 5
      oldpos=motorRotate.get_position()
      while True:
         motorRotate.run_for_degrees(stepsize * self.gearsRotate, speed=30, blocking=True)
         newpos=motorRotate.get_position()
         moved = abs(newpos-oldpos) / self.gearsRotate
         if moved < 4:
            break
         oldpos=newpos
      motorRotate.run_for_degrees(stepsize * self.gearsRotate, speed=-30, blocking=True)
      self.position = 0


   def rotatePosition(self):
      if self.function == 0:
         motorFunction.run_for_degrees(220, speed=-30, blocking=True)
         self.function = 1


   def turnPosition(self):
      if self.function == 1:
         motorFunction.run_for_degrees(230, speed=30, blocking=True)
         self.function = 0


   def flipcube(self, flipspeed = 30, flipangle = 100):
      oldfunction = self.function
      rubiksCube.turnPosition(self)

      degrees = flipangle * self.gearsFunction
      motorFunction.run_for_degrees(degrees, speed=-flipspeed, blocking=True)
      time.sleep(0.2)
      motorFunction.run_for_degrees(degrees + 5 , speed=flipspeed, blocking=True)
      time.sleep(0.2)

      newfunction = self.function
      if oldfunction != newfunction:
         rubiksCube.rotatePosition(self)


   def rotateTo (self, toPosition):
      oldfunction = self.function
      rubiksCube.rotatePosition(self)

      moves = [ [ 0], [-1],    [-2],    [-3],
                [ 1], [ 0],    [-2, 1], [-2],
                [ 2], [ 2,-1], [ 0],    [-1],
                [ 3], [ 2],    [ 1],    [ 0]]

      fromPosition = self.position
      index = fromPosition * 4 + toPosition
      for move in moves[index]:
         degrees = move * 93 * self.gearsRotate
         motorRotate.run_for_degrees(degrees, speed=30, blocking=True)
      self.position = toPosition

      newfunction = self.function
      if oldfunction != newfunction:
         rubiksCube.turnPosition(self)


   def rotatecw(self):
      to_Position = (self.position - 1 + 4)%4
      rubiksCube.rotateTo(self, to_Position)


   def rotatecc(self):
      to_Position = (self.position + 1)%4
      rubiksCube.rotateTo(self, to_Position)


   def turn(self, turntype):
      oldfunction = self.function

      moves = [ ['r',  2, 't',  1, 'r', -2], ['r',  1, 't', -1, 'r',  3], ['r',  1, 't',  2, 'r', -1]
              , ['r',  1, 't',  1, 'r', -1], [     -1]                  , [      2]
              , [      1]                  , ['r', -1, 't', -1, 'r',  1], [     -2]
              , [     -3]                  , ['r', -2, 't', -1, 'r',  2], ['r', -1, 't', -2, 'r',  1]
              , [      2, 't',  1, 'r', -2], [      1, 't', -1, 'r',  3], [      1, 't',  2, 'r', -1]
              , [      1, 't',  1, 'r', -1], ['t', -1                  ], ['t',  2]
              , ['t',  1]                  , [     -1, 't', -1, 'r',  1], ['t', -2]
              , ['t', -3]                  , [     -2, 't', -1, 'r',  2], [     -1, 't', -2, 'r',  1]]

      if   turntype == 0 or turntype == 'cw':
         turnvalue = 0
      elif turntype == 1 or turntype == 'cc':
         turnvalue = 1
      elif turntype == 2 or turntype == 'ht':
         turnvalue = 2

      index = self.function * 12 + self.position * 3 + turnvalue
      for move in moves[index]:
         if move == 't':
            rubiksCube.turnPosition(self)
         elif move == 'r':
            rubiksCube.rotatePosition(self)
         else:
            degrees = move * -93 * self.gearsRotate
            motorRotate.run_for_degrees(degrees, speed=30, blocking=True)
            self.position = self.position + move

      newfunction = self.function
      if oldfunction != newfunction:
         rubiksCube.rotatePosition(self)


   def F(self, turntype):
      oldfunction = self.function

      rubiksCube.flipcube(self)
      rubiksCube.turn(self, turntype)
      rubiksCube.flipcube(self)
      rubiksCube.flipcube(self)
      rubiksCube.flipcube(self)

      newfunction = self.function
      if oldfunction != newfunction:
         rubiksCube.rotatePosition(self)


   def B(self, turntype):
      oldfunction = self.function

      rubiksCube.flipcube(self)
      rubiksCube.flipcube(self)
      rubiksCube.flipcube(self)
      rubiksCube.turn(self, turntype)
      rubiksCube.flipcube(self)

      newfunction = self.function
      if oldfunction != newfunction:
         rubiksCube.rotatePosition(self)


   def U(self, turntype):
      oldfunction = self.function

      rubiksCube.flipcube(self)
      rubiksCube.flipcube(self)
      rubiksCube.turn(self, turntype)
      rubiksCube.flipcube(self)
      rubiksCube.flipcube(self)


      newfunction = self.function
      if oldfunction != newfunction:
         rubiksCube.rotatePosition(self)


   def D(self, turntype):
      oldfunction = self.function

      rubiksCube.turn(self, turntype)

      newfunction = self.function
      if oldfunction != newfunction:
         rubiksCube.rotatePosition(self)


   def L(self, turntype):
      oldfunction = self.function

      rubiksCube.rotatecc(self)
      rubiksCube.F(self, turntype)
      rubiksCube.rotatecw(self)

      newfunction = self.function
      if oldfunction != newfunction:
         rubiksCube.rotatePosition(self)


   def R(self, turntype):
      oldfunction = self.function

      rubiksCube.rotatecw(self)
      rubiksCube.F(self, turntype)
      rubiksCube.rotatecc(self)

      newfunction = self.function
      if oldfunction != newfunction:
         rubiksCube.rotatePosition(self)


   def runList (self, inputlist):
      for i in inputlist:
         if   i == "F":
            rubiksCube.F(self, 0)
         elif i == "F'":
            rubiksCube.F(self, 1)
         elif i == "F2":
            rubiksCube.F(self, 2)
         elif i == "B":
            rubiksCube.B(self, 0)
         elif i == "B'":
            rubiksCube.B(self, 1)
         elif i == "B2":
            rubiksCube.B(self, 2)
         elif i == "U":
            rubiksCube.U(self, 0)
         elif i == "U'":
            rubiksCube.U(self, 1)
         elif i == "U2":
            rubiksCube.U(self, 2)
         elif i == "D":
            rubiksCube.D(self, 0)
         elif i == "D'":
            rubiksCube.D(self, 1)
         elif i == "D2":
            rubiksCube.D(self, 2)
         elif i == "L":
            rubiksCube.L(self, 0)
         elif i == "L'":
            rubiksCube.L(self, 1)
         elif i == "L2":
            rubiksCube.L(self, 2)
         elif i == "R":
            rubiksCube.R(self, 0)
         elif i == "R'":
            rubiksCube.R(self, 1)
         elif i == "R2":
            rubiksCube.R(self, 2)

